#include <linux/config.h>

#ifdef CONFIG_MEDUSA

#include <linux/smp_lock.h>
#include <linux/module.h>
#include <linux/sched.h>
#include "l3_registry.h"
#include "medusa_constants.h"
#include "object.h"
#include "question.h"
#include "server.h"

static unsigned long server_seq = 0;
static struct wait_queue * questions_wait = NULL;

/* main part of asking - this routine traverses the registered servers */
static int do_ask(struct medusa_comm_question_s * q)
{
	int cont;
	unsigned long seq;
	int retval, i;
	struct wait_queue wait;

	while (1) {
		cont = 0;

		write_lock(&medusa_server_lock);
		seq = server_seq;
		MEDUSA_SERVER_FOREACH(q, i) {
			if (medusa_servers[i]) switch(medusa_servers[i]->status) {
				case MED_SERVER_READY:
					medusa_servers[i]->status = MED_SERVER_BUSY;
					write_unlock(&medusa_server_lock);
					MEDUSA_SERVER_CLR(q, i);
					retval = medusa_servers[i]->ask(q);
					write_lock(&medusa_server_lock);
					medusa_servers[i]->status = MED_SERVER_READY;
					if (retval != MED_ERR) {
#ifdef CONFIG_MEDUSA_PROFILING
						medusa_servers[i]->use_count++;
#endif
						write_unlock(&medusa_server_lock);
						return retval;
					}
					seq++; server_seq++;
					wake_up(&questions_wait);
					break;
				case MED_SERVER_BUSY:
					cont = 1;
					break;
				/* case MED_SERVER_SUSP: */
			}
			if (!cont) {
				write_unlock(&medusa_server_lock);
				return MED_ERR;
			}
			seq ^= server_seq;
			if (!seq) {
				current->state = TASK_UNINTERRUPTIBLE;
				wait = { current, NULL };
				add_wait_queue(&questions_wait, &wait);
				write_unlock(&medusa_server_lock);
				schedule();
				remove_wait_queue(&questions_wait, &wait);
				
			} else
				write_unlock(&medusa_server_lock);
		}
	}
}

/* called from l2 code to ask something */
int medusa_comm_ask(struct medusa_comm_question_type_s * type, ...)
{
	va_list args;
	struct medusa_comm_question_s question;
	struct medusa_comm_object_s ** objp = &(question.obj);

	question.question_type_id = type;
#ifdef CONFIG_MEDUSA_PROFILING
	type->ask_count++;
#endif
	question.args = 0;
	MEDUSA_SERVER_FULL(&question);
	va_start(args, type);
	while ((obj = va_arg(args, struct medusa_comm_object_s *)) != NULL) {
#ifdef CONFIG_MEDUSA_PROFILING
		write_lock(&medusa_class_lock);
		obj->class_id->l2_to_l4++;
		write_unlock(&medusa_class_lock);
#endif
		question.args++;
		*objp = obj;
		objp = &(obj->next);
	}
	*objp = NULL;
	return do_ask(&question)
}

/* to use autenthification from modules */
EXPORT_SYMBOL(medusa_comm_ask);
#endif
