#include <linux/config.h>

#ifdef CONFIG_MEDUSA

#include "medusa_constants.h"
#include "object.h"
#include "question.h"
#include "server.h"

/* variables altered by layer 2 registrations */
struct medusa_comm_class_s * medusa_comm_classes = NULL;
rwlock_t medusa_class_lock = RW_LOCK_UNLOCKED;

struct medusa_comm_question_type_s * medusa_comm_question_types = NULL;
rwlock_t medusa_question_type_lock = RW_LOCK_UNLOCKED;

/* variables altered by layer 4 registrations */
struct medusa_comm_server_s * medusa_servers[MEDUSA_SERVERS_MAX];
rwlock_t medusa_server_lock = RW_LOCK_UNLOCKED;

void medusa_l3_registry_init(void)
{
	int i;

	for(i=0; i<MEDUSA_SERVERS_MAX; i++)
		medusa_servers[i] = NULL;
}

struct medusa_comm_question_type_s * medusa_register_question_type(struct medusa_comm_question_type_s * qt)
{
	printk(KERN_INFO "Medusa [L3/R%d]: question type `%s' registered (id L2/Q)\n",
		MEDUSA_COMM_VERSION, qt->name);
	qt->next = medusa_comm_question_types;
	write_lock(&medusa_question_type_lock);
	medusa_comm_question_types = qt;
	write_unlock(&medusa_question_type_lock);
	return qt;
}

int medusa_register_server(struct medusa_comm_server_s * server)
{
	int i;

	for(i=0; i<MEDUSA_SERVERS_MAX; i++)
		if (!medusa_servers[i]) {
			printk(KERN_INFO "Medusa [L3/R%d]: server `%s' registered (id L4/S%d)\n",
				MEDUSA_COMM_VERSION, server->name, i);
			write_lock(&medusa_server_lock);
			medusa_servers[i] = server;
			write_unlock(&medusa_server_lock);
			return i;
		}
	return -1;
}

void medusa_unregister_server(int slotnr)
{
	printk(KERN_INFO "Medusa [L3/%d]: server `%s' unregistered (id L4/%d)\2Yn",
		MEDUSA_COMM_VERSION, medusa_servers[slotnr]->name, slotnr);
	write_lock(&medusa_server_lock);
	medusa_servers[slotnr] = NULL;
	write_unlock(&medusa_server_lock);
}

EXPORT_SYMBOL(medusa_register_server);
EXPORT_SYMBOL(medusa_unregister_server);
EXPORT_SYMBOL(medusa_servers); /* TODO: export some method instead or don't export this at all */
#endif

