#ifndef _MEDUSA_OBJECT_H
#define _MEDUSA_OBJECT_H

/*
 * Used by l2, l3, l4
 *
 * This file defines
 *		CLASS (defined in l2, used by l3, l4)	medusa_comm_class_s
 *		OBJECT (defined in l2, used by l3, l4)	medusa_comm_object_s
 *		ATTRIBUTE (defined in l2, used by l4)	medusa_comm_attribute_s
 */

#include <linux/config.h>
#include "medusa_constants.h"

/**/
/**/

/* describes one attribute in class (defined in l2 code, used by l4) */
struct medusa_comm_attribute_s {
	char name[MEDUSA_ATTRNAME_MAX];	/* string: attribute name */
	unsigned int type;		/* data type (MED_TYPE_xxx) */
	unsigned long offset;		/* offset of attribute in object */
	unsigned long length;		/* bytes consumed by data */
};

/**/
/**/

/* description of class (defined in l2 code, used by l3, l4) */
struct medusa_comm_class_s {
	/* l3-defined data, filled by register_object */
	struct medusa_comm_class_s * next;
#ifdef CONFIG_MEDUSA_PROFILING
	unsigned long l2_to_l4;
	unsigned long l4_to_l2;
#endif

	/* l2-defined data */
	char name[MEDUSA_CLASSNAME_MAX];	/* string: class name */
	unsigned int object_size;		/* sizeof(object) */
	struct medusa_comm_attribute_s * attr;	/* attributes */
};


/* macro for l2 to fill the l3 part of structure */
#ifdef CONFIG_MEDUSA_PROFILING
#define MEDUSA_COMM_DEFAULT_CLASS_HEADER \
	NULL,	/*                   register_class */ \
	0, 0
#else
#define MEDUSA_COMM_DEFAULT_CLASS_HEADER \
	NULL	/*                   register_class */
#endif


/**/
/**/

/* this is the object header - use this at the beginning of l2 structures */
#define MEDUSA_COMM_OBJECT_HEADER \
	struct medusa_comm_class_s * class_id;	/* class - type of object */ \
	char name[MEDUSA_OBJECTNAME_MAX];	/* string: object name */    \
	struct medusa_comm_object_s * next;	/* chain of objects for l3 */ \
	void * serverdata			/* free space for l4 */

/* used by l3 and l4 to easily access header of l2 structures */
struct medusa_comm_object_s {
	MEDUSA_COMM_OBJECT_HEADER,
	unsigned char data[0]
};

#endif

