#ifndef _MEDUSA_QUESTION_H
#define _MEDUSA_QUESTION_H

/*
 * Used by l2, l3, l4
 *
 * This file defines
 *		QUESTION_TYPE (defined in l2/3, used by l3, l4)
 *			medusa_comm_question_type_s
 *		QUESTION (filled in l3 from l2 data, used by l4)
 *			medusa_comm_question_s
 */

#include <linux/config.h>
#include "medusa_constants.h"
#include "object.h"

/**/
/**/

/* describes question types, defined by l2 code */
/* this should be linked by l3 and used by l4 to define the handler names */
struct medusa_comm_question_type_s {
	/* l3-defined data, filled by register_question_type */
	struct medusa_comm_question_type_s * next;
#ifdef CONFIG_MEDUSA_PROFILING
	unsigned long ask_count;
#endif

	/* l2-defined data */
	char name[MEDUSA_QUESTIONNAME_MAX];
	unsigned long flags;	/* unused. later: accepted responses */
				/* 0 = all known responses - use this value
				   for forward compatibility */
}

/* macro for l2 to fill the l3 part of structure */
#ifdef CONFIG_MEDUSA_PROFILING
#define MEDUSA_COMM_DEFAULT_QUESTION_TYPE_HEADER \
	NULL,	/* will be filled by register_question_type */ \
	0	/* ask_count */
#else
#define MEDUSA_COMM_DEFAULT_QUESTION_TYPE_HEADER \
	NULL
#endif

/**/
/**/

/* describes one question from l2 to l4 */
struct medusa_comm_question_s {
	/* l3-filled data */
	struct medusa_comm_question_type_s * question_type_id;
	int args; /* number of objects, comming with a question - set by l3 */
	struct medusa_comm_object_s * obj; /* chained list of objects for question - set by l3 */
	MEDUSA_SERVER_BITMAP; /* bitmap of not-asked servers */
};


#endif

